@@ -6,15 +6,17 @@ Basic wrapper for the Aladhan prayer times API.
 
 from collections import namedtuple
 
-VersionInfo = namedtuple("VersionInfo", "major minor micro releaselevel serial")
+VersionInfo = namedtuple(
+    "VersionInfo", "major minor micro releaselevel serial"
+)
 
 version_info = VersionInfo(
-    major=1, minor=0, micro=0, releaselevel="", serial=0
+    major=1, minor=1, micro=0, releaselevel="final", serial=0
 )
 
 __title__ = "aladhan.py"
 __author__ = "HETHAT"
-__version__ = "1.0.0"
+__version__ = "1.1.0"
 
 
 from .client import Client
