@@ -7,6 +7,7 @@ from .base_types import (
     CalendarDateArg,
     Qibla,
     Ism,
+    Date,
 )
 from .http import HTTPClient
 
@@ -23,6 +24,8 @@ _Calendar = Union[List[Timings], Dict[str, Timings]]
 CalendarR = Union[_Calendar, Awaitable[_Calendar]]
 QiblaR = Union[Qibla, Awaitable[Qibla]]
 AsmaR = Union[List[Ism], Awaitable[List[Ism]]]
+DateR = Union[Date, Awaitable[Date]]
+LDateR = Union[List[Date], Awaitable[List[Date]]]
 
 __all__ = ("Client",)
 
@@ -31,7 +34,8 @@ class Client:
     """
     Al-adhan API client.
 
-    Set to synchronous usage by default, set is_async to True if asynchronous usage wanted.
+    Set to synchronous usage by default, set is_async to True
+        if asynchronous usage wanted.
 
     Synchronous example
 
@@ -65,7 +69,8 @@ class Client:
         loop.run_until_complete(main())
 
     .. note::
-        For Asynchronous usage you need to initialize this class in a |coroutine_link|_.
+        For Asynchronous usage you need to initialize
+            this class in a |coroutine_link|_.
     """
 
     __slots__ = "converter", "http"
@@ -83,7 +88,7 @@ class Client:
         return self.http.is_async
 
     def __enter__(self):
-        if self.is_async:
+        if self.is_async:  # pragma: no cover
             raise TypeError(
                 "Asynchronous client must be used in an asynchronous context"
                 "manager (async with) not in a synchronous one (with)."
@@ -94,7 +99,7 @@ class Client:
         self.close()
 
     async def __aenter__(self):
-        if not self.is_async:
+        if not self.is_async:  # pragma: no cover
             raise TypeError(
                 "Synchronous client must be used in a synchronous context"
                 "manager (with) not in an asynchronous one (async with)."
@@ -138,17 +143,12 @@ class Client:
         ------
             :exc:`~aladhan.exceptions.BadRequest`
                 Invalid parameter was passed.
-
-            :exc:`~aladhan.exceptions.InternalServerError`
         """
-        parameters = {
-            "longitude": str(longitude),
-            "latitude": str(latitude),
-        }
-        date, params = date or TimingsDateArg(), params or Parameters()
-        parameters.update(params.as_dict)
+        date = (date or TimingsDateArg()).date
+        params = (params or Parameters()).as_dict
+        params.update(dict(longitude=str(longitude), latitude=str(latitude)))
         return self.converter.to_timings(
-            self, self.http.get_timings(date.date, parameters)
+            self, self.http.get_timings(date, params)
         )
 
     def get_timings_by_address(
@@ -182,16 +182,12 @@ class Client:
         ------
             :exc:`~aladhan.exceptions.BadRequest`
                 Invalid parameter was passed.
-
-            :exc:`~aladhan.exceptions.InternalServerError`
         """
-        parameters = {
-            "address": address,
-        }
-        date, params = date or TimingsDateArg(), params or Parameters()
-        parameters.update(params.as_dict)
+        date = (date or TimingsDateArg()).date
+        params = (params or Parameters()).as_dict
+        params.update(dict(address=address))
         return self.converter.to_timings(
-            self, self.http.get_timings_by_address(date.date, parameters)
+            self, self.http.get_timings_by_address(date, params)
         )
 
     def get_timings_by_city(
@@ -235,20 +231,14 @@ class Client:
         ------
             :exc:`~aladhan.exceptions.BadRequest`
                 Invalid parameter was passed.
-
-            :exc:`~aladhan.exceptions.InternalServerError`
         """
-        parameters = {
-            "city": city,
-            "country": country,
-            "state": state,
-        }
+        date = (date or TimingsDateArg()).date
+        params = (params or Parameters()).as_dict
+        params.update(dict(city=city, country=country, state=state))
         if state is None:
-            del parameters["state"]
-        date, params = date or TimingsDateArg(), params or Parameters()
-        parameters.update(params.as_dict)
+            del params["state"]
         return self.converter.to_timings(
-            self, self.http.get_timings_by_city(date.date, parameters)
+            self, self.http.get_timings_by_city(date, params)
         )
 
     def get_calendar(
@@ -280,25 +270,19 @@ class Client:
         -------
             :class:`list` of :class:`Timings` or dict[:class:`str`, \
             :class:`list` of :class:`Timings`]
-                A month calendar if month parameter was given in date argument \
-                otherwise a year calendar.
+                A month calendar if month parameter was given in date
+                argument otherwise a year calendar.
 
         Raises
         ------
             :exc:`~aladhan.exceptions.BadRequest`
                 Invalid parameter was passed.
-
-            :exc:`~aladhan.exceptions.InternalServerError`
         """
-        parameters = {
-            "longitude": str(longitude),
-            "latitude": str(latitude),
-        }
-        params = params or Parameters()
-        parameters.update(params.as_dict)
-        parameters.update(date.as_dict)
+        params = (params or Parameters()).as_dict
+        params.update(longitude=str(longitude), latitude=str(latitude))
+        params.update(date.as_dict)
         return self.converter.to_timings(
-            self, self.http.get_calendar(parameters, date.hijri)
+            self, self.http.get_calendar(params, date.hijri)
         )
 
     def get_calendar_by_address(
@@ -308,7 +292,8 @@ class Client:
         params: Optional[Parameters] = None,
     ) -> CalendarR:
         """
-        Get all prayer times for a specific calendar month/year from address.
+        Get all prayer times for a specific calendar month/year
+            from address.
 
         Parameters
         -----------
@@ -326,23 +311,20 @@ class Client:
         -------
             :class:`list` of :class:`Timings` or dict[:class:`str`, \
             :class:`list` of :class:`Timings`]
-                A month calendar if month parameter was given in date argument \
-                otherwise a year calendar.
+                A month calendar if month parameter was given in date
+                argument otherwise a year calendar.
 
         Raises
         ------
             :exc:`~aladhan.exceptions.BadRequest`
                 Invalid parameter was passed.
-
-            :exc:`~aladhan.exceptions.InternalServerError`
         """
-        parameters = {"address": address}
-        params = params or Parameters()
-        parameters.update(params.as_dict)
-        parameters.update(date.as_dict)
+        params = (params or Parameters()).as_dict
+        params.update(dict(address=address))
+        params.update(date.as_dict)
         return self.converter.to_timings(
             self,
-            self.http.get_calendar_by_address(parameters, date.hijri),
+            self.http.get_calendar_by_address(params, date.hijri),
         )
 
     def get_calendar_by_city(
@@ -354,10 +336,11 @@ class Client:
         params: Optional[Parameters] = None,
     ) -> CalendarR:
         """
-        Get all prayer times for a specific calendar month/year from address.
+        Get all prayer times for a specific calendar month/year
+            from address.
 
         Parameters
-        -----------
+        ----------
             city: :class:`str`
                 The city name.
                 Example: "London"
@@ -380,28 +363,21 @@ class Client:
         -------
             :class:`list` of :class:`Timings` or dict[:class:`str`, \
             :class:`list` of :class:`Timings`]
-                A month calendar if month parameter was given in date argument \
-                otherwise a year calendar.
+                A month calendar if month parameter was given in date
+                argument otherwise a year calendar.
 
         Raises
         ------
             :exc:`~aladhan.exceptions.BadRequest`
                 Invalid parameter was passed.
-
-            :exc:`~aladhan.exceptions.InternalServerError`
         """
-        parameters = {
-            "city": city,
-            "country": country,
-            "state": state,
-        }
+        params = (params or Parameters()).as_dict
+        params.update(dict(city=city, country=country, state=state))
         if state is None:
-            del parameters["state"]
-        params = params or Parameters()
-        parameters.update(params.as_dict)
-        parameters.update(date.as_dict)
+            del params["state"]
+        params.update(date.as_dict)
         return self.converter.to_timings(
-            self, self.http.get_calendar_by_city(parameters, date.hijri)
+            self, self.http.get_calendar_by_city(params, date.hijri)
         )
 
     @staticmethod
@@ -422,6 +398,14 @@ class Client:
         """
         Get the Qibla direction from a pair of coordinates.
 
+        Parameters
+        ----------
+            longitude: :class:`int` or :class:`float`
+                Longitude co-ordinate.
+
+            latitude: :class:`int` or :class:`float`
+                Latitude co-ordinate.
+
         Returns
         -------
             :class:`Qibla`
@@ -432,18 +416,21 @@ class Client:
             :exc:`~aladhan.exceptions.BadRequest`
                 Invalid parameter was passed.
 
-            :exc:`~aladhan.exceptions.InternalServerError`
-
         *New in v0.1.3*
         """
         return self.converter.to_qibla(
-            self.http.get_qibla(str(latitude), str(longitude))
+            self.http.get_qibla((latitude, longitude))
         )
 
     def get_asma(self, *n: int) -> AsmaR:
         """
         Returns a list of asma from giving numbers.
 
+        Parameters
+        ----------
+            n: :class:`int`
+                Numbers from range 1-99.
+
         Returns
         -------
             :class:`list` of :class:`Ism`
@@ -454,13 +441,13 @@ class Client:
             :exc:`~aladhan.exceptions.BadRequest`
                 Invalid parameter was passed.
 
-            :exc:`~aladhan.exceptions.InternalServerError`
-
         *New in v0.1.3*
         """
 
         assert n, "No arguments was passed."
-        return self.converter.to_asma(self.http.get_asma(",".join(map(str, n))))
+        return self.converter.to_asma(
+            self.http.get_asma(",".join(map(str, n)))
+        )
 
     def get_all_asma(self) -> AsmaR:
         """
@@ -471,14 +458,161 @@ class Client:
             :class:`list` of :class:`Ism`
                 A list of all asma.
 
-        Raises
-        ------
-            :exc:`~aladhan.exceptions.InternalServerError`
-
         *New in v0.1.3*
         """
         return self.get_asma(*range(1, 100))
 
+    def get_hijri_from_gregorian(
+        self, date: Optional[TimingsDateArg] = None, adjustment: int = 0
+    ) -> DateR:
+        """
+        Convert a gregorian date to a hijri date.
+
+        Parameters
+        ----------
+            date: Optional[:class:`TimingsDateArg`]
+                Gregorian date.
+                Default: Current date
+
+            adjustment: Optional[:class:`int`]
+                Number of days to adjust.
+                Default: 0
+
+        Returns
+        -------
+            :class:`Date`
+                Date in hijri.
+
+        Raises
+        ------
+            :exc:`~aladhan.exceptions.BadRequest`
+                Invalid date or unable to convert it.
+
+        *New in v1.1.0*
+        """
+        params = dict(date=date.date, adjustment=adjustment)
+        return self.converter.to_date(
+            self.http.get_hijri_from_gregorian(params)
+        )
+
+    def get_gregorian_from_hijri(
+        self, date: TimingsDateArg, adjustment: int = 0
+    ) -> DateR:
+        """
+        Convert a hijri date to a gregorian date.
+
+        Parameters
+        ----------
+            date: Optional[:class:`TimingsDateArg`]
+                Gregorian date.
+                Default: Current date
+
+            adjustment: Optional[:class:`int`]
+                Number of days to adjust.
+                Default: 0
+
+        Returns
+        -------
+            :class:`Date`
+                Date in gregorian.
+
+        Raises
+        ------
+            :exc:`~aladhan.exceptions.BadRequest`
+                Invalid date or unable to convert it.
+
+        *New in v1.1.0*
+        """
+        params = dict(date=date.date, adjustment=adjustment)
+        return self.converter.to_date(
+            self.http.get_gregorian_from_hijri(params)
+        )
+
+    def get_hijri_calendar_from_gregorian(
+        self, month: int, year: int, adjustment: int = 0
+    ) -> LDateR:
+        """
+        Get a hijri calendar for a gregorian month.
+
+        Parameters
+        ----------
+            month: :class:`int`
+                Gregorian month.
+
+            year: :class:`int`
+                Gregorian year.
+
+            adjustment: Optional[:class:`int`]
+                Number of days to adjust.
+                Default: 0
+
+        Returns
+        -------
+            :class:`list` of :class:`Date`
+                Hijri Calendar.
+
+        *New in v1.1.0*
+        """
+        return self.converter.to_calendar(
+            self.http.get_hijri_calendar_from_gregorian(
+                (month, year, adjustment)
+            )
+        )
+
+    def get_gregorian_calendar_from_hijri(
+        self, month: int, year: int, adjustment: int = 0
+    ) -> LDateR:
+        """
+        Get a gregorian calendar for a hijri month.
+
+        Parameters
+        ----------
+            month: :class:`int`
+                Hijri month.
+
+            year: :class:`int`
+                Hijri year.
+
+            adjustment: Optional[:class:`int`]
+                Number of days to adjust.
+                Default: 0
+
+        Returns
+        -------
+            :class:`list` of :class:`Date`
+                Gregorian Calendar.
+
+        *New in v1.1.0*
+        """
+        return self.converter.to_calendar(
+            self.http.get_gregorian_calendar_from_hijri(
+                (month, year, adjustment)
+            )
+        )
+
+    def get_islamic_year_from_gregorian_for_ramadan(self, year: int) -> int:
+        """
+        Get which islamic year for ramadan from a gregorian year.
+
+        Parameters
+        ----------
+            year: :class:`int`
+                Gregorian year.
+
+        Returns
+        -------
+            :class:`int`
+                Hijri year.
+
+        Raises
+        ------
+            :exc:`~aladhan.exceptions.BadRequest`
+                Unable to compute year.
+        """
+        return self.converter.to_int(
+            self.http.get_islamic_year_from_gregorian_for_ramadan(year)
+        )
+
 
 class _SyncConverter:
     @staticmethod
@@ -493,7 +627,9 @@ class _SyncConverter:
                 month: [Data(**day, client=client).timings for day in days]
                 for month, days in data.items()
             }
-        return Data(**data, client=client).timings  # it is just a day timings
+
+        # it is just a day timings
+        return Data(**data, client=client).timings
 
     @staticmethod
     def to_qibla(o):
@@ -503,6 +639,18 @@ class _SyncConverter:
     def to_asma(o):
         return [Ism(**d) for d in o]
 
+    @staticmethod
+    def to_date(o):
+        return Date(**o)
+
+    @staticmethod
+    def to_calendar(o):
+        return [Date(**d) for d in o]
+
+    @staticmethod
+    def to_int(o):
+        return int(o)
+
 
 class _AsyncConverter:
     @staticmethod
@@ -517,7 +665,9 @@ class _AsyncConverter:
                 month: [Data(**day, client=client).timings for day in days]
                 for month, days in data.items()
             }
-        return Data(**data, client=client).timings  # it is just a day timings
+
+        # it is just a day timings
+        return Data(**data, client=client).timings
 
     @staticmethod
     async def to_qibla(o):
@@ -526,3 +676,15 @@ class _AsyncConverter:
     @staticmethod
     async def to_asma(o):
         return [Ism(**d) for d in await o]
+
+    @staticmethod
+    async def to_date(o):
+        return Date(**(await o))
+
+    @staticmethod
+    async def to_calendar(o):
+        return [Date(**d) for d in await o]
+
+    @staticmethod
+    async def to_int(o):
+        return int(await o)
